﻿''
'' WindowsFormsVB
''
'' Copyright (c) 2016 http://www.curict.com
'' This software is released under the MIT License, see LICENSE.txt.
''

Public Class EnterMoveDataGridView
    ''' <summary>
    ''' ダイアログ ボックスのキーを処理します。
    ''' </summary>
    Protected Overrides Function ProcessDialogKey(ByVal keyData As Keys) As Boolean
        ' セルの編集モード時にEnterが押されると次行に移ってしまうので、右隣のセルに移動させる
        If (keyData And Keys.KeyCode) = Keys.Enter Then
            ' Tabキーの処理を行う
            Return Me.ProcessTabKey(keyData)
        End If

        ' 既定の処理を行う
        Return MyBase.ProcessDialogKey(keyData)
    End Function

    ''' <summary>
    ''' DataGridView での移動に使用されるキーを処理します。
    ''' </summary>
    Protected Overrides Function ProcessDataGridViewKey(ByVal e As KeyEventArgs) As Boolean
        If e.KeyCode = Keys.Enter And e.Handled = False Then
            ' イベントを処理済にする
            e.Handled = True

            If Me.CurrentCell IsNot Nothing Then
                ' 右下セルのときは次のコントロールにフォーカス移動
                If Me.CurrentCell.RowIndex = Me.Rows.GetLastRow(DataGridViewElementStates.Visible) AndAlso _
                   Me.CurrentCell.ColumnIndex = Me.Columns.GetLastColumn(DataGridViewElementStates.Visible, DataGridViewElementStates.None).Index AndAlso _
                   e.Modifiers <> Keys.Shift Then
                    Return Me.FindForm().SelectNextControl(Me.FindForm().ActiveControl, True, True, True, True)
                End If

                ' 左上のセルでShift + Enterが押されたときは前のコントロールにフォーカス移動
                If Me.CurrentCell.RowIndex = 0 AndAlso _
                   Me.CurrentCell.ColumnIndex = 0 AndAlso _
                   e.Modifiers = Keys.Shift Then
                    Return Me.FindForm().SelectNextControl(Me.FindForm().ActiveControl, False, True, True, True)
                End If
            End If

            ' Enterキーが押されらTabキーの処理を行う
            Return Me.ProcessTabKey(e.KeyData)
        End If

        ' 既定の処理を行う
        Return MyBase.ProcessDataGridViewKey(e)
    End Function
End Class
